/*!
  \file libebs.c
  \date 19.08.2002
  \version 0.1.1
  \author robert.sedevcic@ijs.si
  \brief This is EBS-Experimental Boundary Scan project library source file.
  Issue ./source/share_lib_gen command to create and install library. This library
  contains some hardware related functions.

  See EBS project home page at http://ebsp.sourceforge.net

  Copyright (C) 2002 by CSD at http://www-csd.ijs.si
*/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <comedi.h>
#include <ebs.h>
#include <unistd.h>


/*=============================================================================
 * general purpose function definitions
 *===========================================================================*/
/*!
  \fn int hextouc(unsigned char* cptr, int mode)
  \brief The hextouc() converts the groups of 4 hexa characters (4 bytes
  representing 16-bit value) into binary format (2 bytes = 16 bits), stored
  back into the same string. If number of characters is not multiple of 4,
  then 0 are left padded.\n
  Example of transformation (4x8->2x8):\n
        1. Before transformation:  buff="1AB0"...takes 32 bits\n
	2. After transformation if mode=0:\n
	                         *buff=176 dec...takes 8 bits \n
	                       *(buf+1)=26 dec...takes 8 bits\n
	   After transformation if mode=1:\n
	                          *buff=26 dec...takes 8 bits \n
	                      *(buf+1)=176 dec...takes 8 bits\n
  Or another example:
  Character string "C1AB0" is transformed as; first three zeroes are
  padded on the left ("000C1AB0"). Then two characters are combined
  together (00, 0C, 1A, B0) and transformed into binary values (00=0,
  0C=12, 1A=26, B0=176). Those values are swaped (12, 0, 176, 26). This
  takes 4 memory bytes (write(buff, 4);). At shifting (LSB):\n
  0000 1100 0001 1010 1011 0000 ->(first out) \n
   0    C    1    A    B    0\n

  \param cptr - character pointer
  \param mode - mode=0 data, mode=1 mask
  \return int - number of bytes (multiple of 2) if ok, -1 otherwise
*/
int hextouc(unsigned char* cptr, int mode)
{
  char *tmpstr;               /* temporary string holder */
  size_t len, nlen;           /* original & padded string length */
  int cnt = 0;                /* general counter - from 0 to sizeof(*cptr) */
  unsigned int i = 0;         /* local counter when saving back to buffer */
  short int shift_no = 4;     /* nibble bit-width */
  unsigned short tmp_shi = 0; /* temporary integer value of characters */
  
  len = strlen((char*) cptr);  /* length of input string without '\0' */
  nlen = len;

  while (nlen % shift_no) {  /* increase to a multiple of 4 */
    nlen++;
  }

  tmpstr = (char*) malloc(nlen + 1);  /* allocate tmp string */

  while ((nlen - i) > len) {  /* left pad with '0's */
    *(tmpstr + i) = '0';
    i++;
  }

  strcpy(tmpstr + i, (char*) cptr); /* copy original to temp. string */

  memset(cptr, 0, sizeof(*cptr) * (len + 1));  /* clear original string */
  i = 0;  /* reset counter */
	
  while(*(tmpstr + cnt)) {
    tmp_shi <<= shift_no;
    switch(*(tmpstr + cnt)) {
      case 'A':
      case 'B':
      case 'C':
      case 'D':
      case 'E':
      case 'F':
        tmp_shi |= ((int)*(tmpstr + cnt) - 55);
        break;
      case 'a':
      case 'b':
      case 'c':
      case 'd':
      case 'e':
      case 'f':
        tmp_shi |= ((int)*(tmpstr + cnt) - 87);
        break;
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        tmp_shi |= ((int)*(tmpstr + cnt) - 48);
        break;
      default:
	return -1;
    };
    cnt++;
    if (!(cnt % shift_no)) {
      if (mode) {
	*(cptr + i) = tmp_shi >> 8;
      }
      else {
	*(cptr + i) = tmp_shi;
      }
      i++;
      if (mode) {
	*(cptr + i) = tmp_shi;
      }
      else {
	*(cptr + i) = tmp_shi >> 8;
      }
      i++;
      tmp_shi = 0;
      }
  }

  if (cnt % shift_no) {
    while (cnt % shift_no) {
      tmp_shi <<= shift_no;
      cnt++;
    }
    if (mode) {
      *(cptr + i) = tmp_shi >> 8;
    }
    else {
    *(cptr + i) = tmp_shi;
    }
    i++;
    if (mode) {
      *(cptr + i) = tmp_shi;
    }
    else {
      *(cptr + i) = tmp_shi >> 8;
    }
    i++;
  }

  *(cptr + i) = '\0';  /* redundant, just to be sure */

  free(tmpstr);
  
  return (i);
}//end of hextouc()


/*!
  \fn void err_check(int ret, int line)
  \brief The err_check() checks if any error happened in the application 
  which operates directly on the driver. It prints error message to the
  stderr and exits.
  \param ret  - returned value
  \param line - line of error
*/
void err_check(int ret, int line)
{
  if ( ret < 0) {
    fprintf(stderr, "EBS applic: %s, error on line %d\n",
	    strerror(errno), line - 1);
    exit(0);
  }
}//end of err_check(int ret, int line)


/*!
  \fn unsigned char *binary_to_hex(int bytes_no, unsigned char *cptr)
  \brief The binary_to_hex() transforms binary values written in cptr
  to the associate characters.
  \param bytes_no  - bytes number
  \param cptr - character pointer
  \return character pointer
*/
unsigned char *binary_to_hex(int bytes_no, unsigned char *cptr)
{
  int i,j;
  unsigned char *tmp;

  /* allocate space */
  if (!(tmp = (char *) malloc((bytes_no*2) + 1))) {
    fprintf(stderr,"error: %s\n", strerror(errno));
    exit(0);
  }

  for (i=0,j=0; j < bytes_no; i++,j++) {
    *(tmp+i) = (*(cptr+j) >> 4);
    if (*(tmp + i) < 10) {
      *(tmp + i) += 48;
    }
    else {
      *(tmp + i) += 55;
    }
    i++;

    *(tmp+i) = (*(cptr+j) & 0x0F);
    if (*(tmp + i) < 10) {
      *(tmp + i) += 48;
    }
    else {
      *(tmp + i) += 55;
    }
  }

  *(tmp+i)='\0';

  return tmp;
}//end of binary_to_hex(int bytes_no, unsigned char *cptr)






